/* eslint-disable no-undef */
import { createAsyncThunk, createSlice } from '@reduxjs/toolkit';
import { groupEdges, getNodeType } from './utils';

const initialState = {
  nodes: [],
  edges: [],
  animating: false,
  nodeGroupingEnabled: false,
  edgeGroupingEnabled: false,
  geoEnabled: false,
  geoClusteringEnabled: false
};

export const setGraph = createAsyncThunk('ogma/setGraph', graph => {
  return ogma.parse
    .neo4j(graph)
    .then(graph => {
      return ogma.setGraph(graph);
    })
    .then(() => ogma.layouts.force({ locate: true }))
    .then(() => ogma.export.json({ download: false }));
});

export const toggleNodeGrouping = createAsyncThunk(
  'ogma/toggleNodeGrouping',
  () => {
    let grouping = ogma.transformations
      .getList()
      .find(t => t.getName() === 'node-grouping');

    return Promise.resolve()
      .then(() => {
        if (grouping) {
          return grouping;
        }
        return ogma.transformations.addNodeGrouping({
          groupIdFunction: function (node) {
            return getNodeType(node);
          },
          selector: function (node) {
            return getNodeType(node) !== 'Product';
          },
          enabled: false,
          edgeGenerator: function (edges) {
            return groupEdges(edges);
          },
          nodeGenerator: function (nodes, groupId) {
            // choose label
            let label = ['Wholesalers']; // default
            if (groupId === 'SupplierA') label = ['FrameSupplier'];
            else if (groupId === 'SupplierB') label = ['WheelSupplier'];
            else if (/RawSupplier/.test(groupId)) label = ['RawSuppliers'];
            else if (groupId === 'Retailer') label = ['Retailers'];

            // generated group node
            return {
              data: {
                neo4jLabels: label
              }
            };
          }
        });
      })
      .then(g => {
        grouping = g;
        return grouping.toggle();
      })
      .then(() => {
        if (grouping.isEnabled()) {
          return ogma.layouts.hierarchical({
            componentDistance: 500,
            nodeDistance: 50,
            levelDistance: 80,
            locate: true,
            direction: 'LR'
          });
        } else {
          return ogma.layouts.force({ locate: true });
        }
      })
      .then(() => grouping.isEnabled());
  }
);

export const toggleGeo = createAsyncThunk(
  'ogma/toggleGeo',
  (args, { getState }) => {
    return ogma.geo
      .toggle({
        longitudePath: 'neo4jProperties.lon',
        latitudePath: 'neo4jProperties.lat'
      })
      .then(() => {
        if (ogma.geo.enabled()) {
          return Promise.resolve();
        }
        const state = getState();
        if (state.ogma.nodeGroupingEnabled) {
          return ogma.layouts.hierarchical({
            componentDistance: 500,
            nodeDistance: 50,
            levelDistance: 80,
            direction: 'LR',
            locate: true
          });
        }
        return ogma.layouts.force({ locate: true });
      })

      .then(() => ogma.geo.enabled());
  }
);

export const toggleEdgeGrouping = createAsyncThunk(
  'ogma/toggleEdgeGrouping',
  () => {
    let grouping = ogma.transformations
      .getList()
      .find(t => t.getName() === 'edge-grouping');

    return Promise.resolve()
      .then(() => {
        if (grouping) {
          return grouping;
        }
        return ogma.transformations.addEdgeGrouping({
          selector: function (edge) {
            return edge.getData('neo4jType') === 'DELIVER';
          },
          enabled: false,
          generator: function (edges) {
            return groupEdges(edges);
          }
        });
      })
      .then(g => {
        grouping = g;
        return grouping.toggle();
      })
      .then(() => ogma.layouts.force({ locate: true }))
      .then(() => grouping.isEnabled());
  }
);

export const toggleGeoClustering = createAsyncThunk(
  'ogma/toggleGeoClustering',
  () => {
    const grouping = ogma.transformations
      .getList()
      .find(t => t.getName() === 'geo-clustering');

    return Promise.resolve()
      .then(() => {
        if (grouping) {
          return grouping;
        }
        return ogma.transformations.addGeoClustering({
          groupIdFunction: function (node) {
            const type = getNodeType(node);
            return /RawSupplier/.test(type) ? 'RawSuppliers' : type;
          },
          selector: function (node) {
            return getNodeType(node) !== 'Product';
          },
          enabled: true,
          edgeGenerator: function (edges) {
            return groupEdges(edges);
          },
          nodeGenerator: function (nodes, groupId) {
            // choose label
            let label = ['Wholesalers']; // default
            if (/RawSupplier/.test(groupId)) label = ['RawSuppliers'];
            else if (/SupplierA/.test(groupId)) label = ['SupplierA'];
            else if (/SupplierB/.test(groupId)) label = ['SupplierB'];
            else if (/Retailer/.test(groupId)) label = ['Retailer'];

            // generated group node
            return {
              data: {
                neo4jLabels: label
              },
              attributes: {
                badges: {
                  bottomRight:
                    nodes.size > 1
                      ? { text: { content: nodes.size } }
                      : undefined
                }
              }
            };
          }
        });
      })
      .then(grouping => grouping.isEnabled());
  }
);

export const updateNodes = createAsyncThunk('ogma/updateNodes', () => {
  return ogma.export.json({ download: false });
});

export const ogmaSlice = createSlice({
  name: 'ogma',
  initialState,
  reducers: {},
  extraReducers: builder => {
    builder.addCase(setGraph.fulfilled, (state, action) => {
      const { nodes, edges } = action.payload;
      state.nodes = nodes;
      state.edges = edges;
      state.animating = false;
    });
    builder.addCase(setGraph.pending, (state, action) => {
      state.animating = true;
    });
    builder.addCase(setGraph.rejected, (state, action) => {
      state.animating = false;
    });
    builder.addCase(toggleNodeGrouping.fulfilled, (state, action) => {
      state.animating = false;
      state.nodeGroupingEnabled = action.payload;
    });
    builder.addCase(toggleNodeGrouping.pending, (state, action) => {
      state.animating = true;
    });
    builder.addCase(toggleNodeGrouping.rejected, (state, action) => {
      state.animating = false;
    });
    builder.addCase(toggleEdgeGrouping.fulfilled, (state, action) => {
      state.animating = false;
      state.edgeGroupingEnabled = action.payload;
    });
    builder.addCase(toggleEdgeGrouping.pending, (state, action) => {
      state.animating = true;
    });
    builder.addCase(toggleEdgeGrouping.rejected, (state, action) => {
      state.animating = false;
    });
    builder.addCase(toggleGeo.fulfilled, (state, action) => {
      state.animating = false;
      state.geoEnabled = action.payload;
    });
    builder.addCase(toggleGeo.pending, (state, action) => {
      state.animating = true;
    });
    builder.addCase(toggleGeo.rejected, (state, action) => {
      state.animating = false;
    });

    builder.addCase(toggleGeoClustering.fulfilled, (state, action) => {
      state.animating = false;
      state.geoClusteringEnabled = action.payload;
    });
    builder.addCase(toggleGeoClustering.pending, (state, action) => {
      state.animating = true;
    });
    builder.addCase(toggleGeoClustering.rejected, (state, action) => {
      state.animating = false;
    });
    builder.addCase(updateNodes.fulfilled, (state, action) => {
      state.nodes = action.payload.nodes;
    });
  }
});

export const ogmaSelector = state => state.ogma;

export const ogmaReducer = ogmaSlice.reducer;
