import Ogma from '@linkurious/ogma';
import React, { PureComponent } from 'react';
import { shouldPulse, getNodeType } from './utils';

function createStyles(ogma) {
  // Define buckets of importance for nodes
  // some values repeat themselves for different types
  const supplierStyle = { color: '#6FC1A7', icon: '\uf275' };
  const rawSupplierStyle = { color: '#3DCB8D', icon: '\uf472' };
  const wholeSalerStyle = { color: '#E77152', icon: '\uf468' };
  const retailerStyle = { color: '#9AD0D0', icon: '\uf07a' };

  const nodeStyles = {
    SupplierA: supplierStyle,
    SupplierB: supplierStyle,

    RawSupplierA: rawSupplierStyle,
    RawSupplierB: rawSupplierStyle,
    RawSuppliers: rawSupplierStyle,

    Retailer: retailerStyle,
    Retailers: retailerStyle,

    Wholesaler: wholeSalerStyle,
    Wholesalers: wholeSalerStyle,

    FrameSupplier: { color: '#6FC1A7', icon: '\uf565' },
    WheelSupplier: { color: '#6FC1A7', icon: '\uf655' },

    Product: { color: '#76378A', icon: '\uf466' },
    unknown: { color: '#777', icon: '' }
  };

  // edge color and width buckets
  const edgeBuckets = ogma.rules.slices({
    field: 'neo4jProperties.quantity',
    values: [
      { color: '#DDD', width: 0.5 },
      { color: '#AAA', width: 1.5 },
      { color: '#777', width: 2 },
      { color: '#444', width: 3.5 },
      { color: '#444', width: 5 }
    ],
    fallback: { color: '#444', width: 3.5 },
    stops: [15, 150, 280, 500]
  });

  ogma.styles.addRule({
    // Edges style:
    edgeAttributes: function (edge) {
      const { color, width } = edgeBuckets(edge) || {
        color: '#444',
        width: 5
      };
      return {
        shape: {
          head: 'arrow'
        },
        color: color,
        width: width
      };
    },
    // Node style:
    nodeAttributes: function (node) {
      const type = getNodeType(node);
      const color = nodeStyles[type].color,
        icon = nodeStyles[type].icon;

      return {
        icon: {
          font: 'Font Awesome 5 Free',
          color: '#000000',
          content: icon,
          minVisibleSize: 0
        },
        // define colors depending on the type of the node
        color: color,
        // nodes size depend on the quantity of Product that is exanged through them
        radius:
          10 +
          node.getAdjacentEdges().reduce(function (acc, e) {
            return acc + e.getData('neo4jProperties.quantity');
          }, 0) /
            300,
        // Make nodes that have a higher demand than their stock pulsating
        pulse: {
          enabled: shouldPulse(node),
          endRatio: 1.5,
          width: 4,
          interval: 600,
          startRatio: 1.0
        }
      };
    }
  });
}

export default class OgmaComponent extends PureComponent {
  componentDidUpdate(prevProps) {
    // add a node
    if (
      prevProps.data &&
      prevProps.data.nodes.length < this.props.data.nodes.length
    ) {
      // update the data in ogma
      return this.ogmaRef
        .addGraph(this.props.data)
        .then(() => this.runLayout());
    }

    if (prevProps.layout !== this.props.layout) {
      return this.runLayout();
    }
  }

  runLayout() {
    const options = { locate: true };
    if (this.props.layout === 'hierarchical') {
      // set node with ID = 1 as root of the hierarchy. Shown as an example to customise options per-layout
      // Alternatively the layout can work out the root automatically, or we could use user's selection.
      options.roots = [1];
    }
    return this.ogmaRef.layouts[this.props.layout](options);
  }

  componentDidMount() {
    // create the Ogma instance
    // eslint-disable-next-line no-undef
    Ogma.libraries['leaflet'] = L;
    this.ogmaRef = new Ogma({
      container: 'ogma-react',
      options: {
        backgroundColor: 'rgb(240, 240, 240)'
      },
      graph: this.props.data
    });
    createStyles(this.ogmaRef);
    this.ogmaRef.layouts.force({ locate: true });
    window.ogma = this.ogmaRef;
    window.Ogma = Ogma;

    this.bindEvents();
  }

  // when passed a on* event as React prop it checks it and bind it to the ogma instance
  bindEvents() {
    // See the Ogma.events namespace: https://doc.linkurio.us/ogma/latest/api/ogma/events
    Object.keys(this.props).forEach(prop => {
      if (/^on/.test(prop) && prop in this.ogmaRef.events) {
        this.ogmaRef.events[prop](this.props[prop]);
      }
    });
  }

  componentWillUnmount() {
    this.ogmaRef.destroy();
  }

  render() {
    return (
      <div id="ogma-react" style={{ width: 800, height: 600 }}>
        {' '}
      </div>
    );
  }
}
